<?php

/**
 * Copyright © 2013-2017 Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\MergeOnContent\Magento\Framework\View\Asset;

/**
 * \Iterator that aggregates one or more assets and provides a single public file with equivalent behavior
 */
class Merged extends \Magento\Framework\View\Asset\Merged {

    protected $helperMerge = false;

    /**
     * @var \Magento\Framework\View\Asset\Repository
     */
    protected $assetRepo;

    /**
     * @var \Magento\Framework\App\View\Deployment\Version
     */
    private $deploymentVersion;
    
    public function __construct(
    \Psr\Log\LoggerInterface $logger, \Magento\Framework\View\Asset\MergeStrategyInterface $mergeStrategy, \Magento\Framework\View\Asset\Repository $assetRepo, array $assets, \FiloBlu\MergeOnContent\Helper\Data $helperMerge, \Magento\Framework\App\View\Deployment\Version $deploymentVersion
    ) {
        $this->helperMerge = $helperMerge;
        $this->deploymentVersion = $deploymentVersion;
        parent::__construct($logger, $mergeStrategy, $assetRepo, $assets);
        $this->assetRepo = $assetRepo;
    }

    protected function initialize() {
        if (!$this->isInitialized) {
            $this->isInitialized = true;
            try {
                $mergedAsset = $this->createMergedAsset($this->assets);
                $this->mergeStrategy->merge($this->assets, $mergedAsset);
                $this->assets = [$mergedAsset];
            } catch (\Exception $e) {
                $this->logger->critical($e);
            }
        }
    }

    /**
     * Create an asset object for merged file
     *
     * @param array $assets
     * @return MergeableInterface
     */
    private function createMergedAsset(array $assets) {

        $versionNumber = ($this->helperMerge->getConfig("general/append_version") ? $this->deploymentVersion->getValue() : "");
        
        if ($this->helperMerge->getConfig("general/enabled")) {

            foreach ($assets as $asset) {
                try {
                    $contentFile .= $asset->getContent();
                } catch (\Exception $ex) {
                    
                }
            }

            $filePath = md5($contentFile) . $versionNumber . '.' . $this->contentType;

            return $this->assetRepo->createArbitrary($filePath, self::getRelativeDir());
        } else {
            $paths = [];
            /** @var MergeableInterface $asset */
            foreach ($assets as $asset) {
                $paths[] = $asset->getPath();
            }
            $paths = array_unique($paths);
            
            $filePath = md5(implode('|', $paths)) . $versionNumber . '.' . $this->contentType;
            return $this->assetRepo->createArbitrary($filePath, self::getRelativeDir());
        }
    }

}
